package kr.test.openid;

import org.openid4java.consumer.ConsumerManager;
import org.openid4java.consumer.ConsumerException;
import org.openid4java.consumer.VerificationResult;
import org.openid4java.discovery.Identifier;
import org.openid4java.discovery.Discovery;
import org.openid4java.discovery.DiscoveryInformation;
import org.openid4java.message.*;
import org.openid4java.server.RealmVerifier;
import org.openid4java.OpenIDException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.io.IOException;
import org.openid4java.message.sreg.SRegRequest;
import org.openid4java.message.sreg.SRegMessage;
import org.openid4java.message.sreg.SRegResponse;


public class SampleConsumer{    
	public ConsumerManager manager;    
	public SampleConsumer() throws ConsumerException    {                
		manager = new ConsumerManager();    
	}   
	
	public String authRequest(String userSuppliedString, HttpServletRequest httpReq, HttpServletResponse httpResp) throws IOException {        
		try {                       
			String returnToUrl = "http://localhost:8090/OpenId/openid/resultOpenid.jsp";   
			String trustRoot  = "http://localhost:8090/OpenId/";
            
			Identifier identifier = Discovery.parseIdentifier(userSuppliedString);
			userSuppliedString = identifier.getIdentifier();

			List discoveries = manager.discover(userSuppliedString);                        
			DiscoveryInformation discovered = manager.associate(discoveries);
			
			RealmVerifier rv = new RealmVerifier();
		    rv.setEnforceRpId(false);
		    manager.setRealmVerifier(rv); 
		    
			httpReq.getSession().setAttribute("openid-disc", discovered);			

			AuthRequest authReq = manager.authenticate(discovered, returnToUrl, trustRoot);
			
			SRegRequest sreq = SRegRequest.createFetchRequest();
			sreq.addAttribute("email", true);
			sreq.addAttribute("nickname", true);
			//sreq.addAttribute("fullname", true);
			sreq.addAttribute("dob", true);
			sreq.addAttribute("gender", true);
			authReq.addExtension(sreq);
         
			if (! discovered.isVersion2() ) {       
				httpResp.sendRedirect(authReq.getDestinationUrl(true));                
				return null;            
			} else {                
			}        
		} catch (OpenIDException e) {           
			// present error to the user        
		}        
		return null;    
	}    
	
	public Identifier verifyResponse(HttpServletRequest httpReq) {        
		try {            
			            
			ParameterList response = new ParameterList(httpReq.getParameterMap());            
			            
			DiscoveryInformation discovered = (DiscoveryInformation)httpReq.getSession().getAttribute("openid-disc");            
			            
			StringBuffer receivingURL = httpReq.getRequestURL();            
			String queryString = httpReq.getQueryString();            
			
			if (queryString != null && queryString.length() > 0) {
				receivingURL.append("?").append(httpReq.getQueryString());            
			}
			            
			VerificationResult verification = manager.verify(receivingURL.toString(), response, discovered);            
			
			Identifier verified = verification.getVerifiedId();            
			
			if (verified != null) {                
				AuthSuccess authSuccess = (AuthSuccess) verification.getAuthResponse();
							
				if (authSuccess.hasExtension(SRegMessage.OPENID_NS_SREG))
				{
					MessageExtension ext = authSuccess.getExtension(SRegMessage.OPENID_NS_SREG);
							    	
					if (ext instanceof SRegResponse)
					{
						SRegResponse sregResp = (SRegResponse) ext;
							        
						String nickName = sregResp.getAttributeValue("nickname");
						String email = sregResp.getAttributeValue("email");
						String dob = sregResp.getAttributeValue("dob");
						String gender = sregResp.getAttributeValue("gender");
							        
						httpReq.setAttribute("nickName", nickName);
						httpReq.setAttribute("email", email);
						httpReq.setAttribute("dob", dob);
						httpReq.setAttribute("gender", gender);
					}
				}

				return verified;  // success            
			}         
		} catch (OpenIDException e) {
			System.out.println("################## : error");
		}        
		return null;    
	}
}